@@ -13,21 +13,12 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclCmdIL.c 1.173 97/11/18 13:55:01
+ * SCCS: @(#) tclCmdIL.c 1.4 98/07/01 18:37:16
  */
 
 #include "tclInt.h"
 #include "tclPort.h"
 
-/*
- * The following variable holds the full path name of the binary
- * from which this application was executed, or NULL if it isn't
- * know.  The value of the variable is set by the procedure
- * Tcl_FindExecutable.  The storage space is dynamically allocated.
- */
-
-char *tclExecutableName = NULL;
-
 /*
  * During execution of the "lsort" command, structures of the following
  * type are used to arrange the objects being sorted into a collection
@@ -81,6 +72,9 @@ typedef struct SortInfo {
  * Forward declarations for procedures defined in this file:
  */
 
+static void		AppendLocals _ANSI_ARGS_((Tcl_Interp *interp,
+			    Tcl_Obj *listPtr, char *pattern,
+			    int includeLinks));
 static int		DictionaryCompare _ANSI_ARGS_((char *left,
 			    char *right));
 static int		InfoArgsCmd _ANSI_ARGS_((ClientData dummy,
@@ -1202,14 +1196,7 @@ InfoLocalsCmd(dummy, interp, objc, objv)
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     Interp *iPtr = (Interp *) interp;
-    Proc *procPtr;
-    CompiledLocal *localPtr;
-    Var *varPtr;
-    char *varName, *pattern;
-    int i, localVarCt;
-    Tcl_HashTable *localVarTablePtr;
-    register Tcl_HashEntry *entryPtr;
-    Tcl_HashSearch search;
+    char *pattern;
     Tcl_Obj *listPtr;
 
     if (objc == 2) {
@@ -1220,11 +1207,10 @@ InfoLocalsCmd(dummy, interp, objc, objv)
         Tcl_WrongNumArgs(interp, 2, objv, "?pattern?");
         return TCL_ERROR;
     }
-
+    
     if (iPtr->varFramePtr == NULL || !iPtr->varFramePtr->isProcCallFrame) {
         return TCL_OK;
     }
-    localVarTablePtr = iPtr->varFramePtr->varTablePtr;
 
     /*
      * Return a list containing names of first the compiled locals (i.e. the
@@ -1233,16 +1219,54 @@ InfoLocalsCmd(dummy, interp, objc, objv)
      */
     
     listPtr = Tcl_NewListObj(0, (Tcl_Obj **) NULL);
-    
-    procPtr = iPtr->varFramePtr->procPtr;
-    localVarCt = procPtr->numCompiledLocals;
-    localPtr = procPtr->firstLocalPtr;
+    AppendLocals(interp, listPtr, pattern, 0);
+    Tcl_SetObjResult(interp, listPtr);
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * AppendLocals --
+ *
+ *	Append the local variables for the current frame to the
+ *	specified list object.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+AppendLocals(interp, listPtr, pattern, includeLinks)
+    Tcl_Interp *interp;		/* Current interpreter. */
+    Tcl_Obj *listPtr;		/* List object to append names to. */
+    char *pattern;		/* Pattern to match against. */
+    int includeLinks;		/* 1 if upvars should be included, else 0. */
+{
+    Interp *iPtr = (Interp *) interp;
+    CompiledLocal *localPtr;
+    Var *varPtr;
+    int i, localVarCt;
+    char *varName;
+    Tcl_HashTable *localVarTablePtr;
+    register Tcl_HashEntry *entryPtr;
+    Tcl_HashSearch search;
+
+    localPtr = iPtr->varFramePtr->procPtr->firstLocalPtr;
+    localVarCt = iPtr->varFramePtr->numCompiledLocals;
     varPtr = iPtr->varFramePtr->compiledLocals;
+    localVarTablePtr = iPtr->varFramePtr->varTablePtr;
 
     for (i = 0; i < localVarCt; i++) {
 	/*
 	 * Skip nameless (temporary) variables and undefined variables
 	 */
+
 	if (!localPtr->isTemp && !TclIsVarUndefined(varPtr)) {
 	    varName = varPtr->name;
 	    if ((pattern == NULL) || Tcl_StringMatch(varName, pattern)) {
@@ -1253,13 +1277,14 @@ InfoLocalsCmd(dummy, interp, objc, objv)
 	varPtr++;
 	localPtr = localPtr->nextPtr;
     }
-
+    
     if (localVarTablePtr != NULL) {
 	for (entryPtr = Tcl_FirstHashEntry(localVarTablePtr, &search);
 	        entryPtr != NULL;
                 entryPtr = Tcl_NextHashEntry(&search)) {
 	    varPtr = (Var *) Tcl_GetHashValue(entryPtr);
-	    if (!TclIsVarUndefined(varPtr) && !TclIsVarLink(varPtr)) {
+	    if (!TclIsVarUndefined(varPtr)
+		    && (includeLinks || !TclIsVarLink(varPtr))) {
 		varName = Tcl_GetHashKey(localVarTablePtr, entryPtr);
 		if ((pattern == NULL)
 		        || Tcl_StringMatch(varName, pattern)) {
@@ -1269,9 +1294,6 @@ InfoLocalsCmd(dummy, interp, objc, objv)
 	    }
 	}
     }
-    
-    Tcl_SetObjResult(interp, listPtr);
-    return TCL_OK;
 }
 
 /*
@@ -1302,13 +1324,17 @@ InfoNameOfExecutableCmd(dummy, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
+    CONST char *nameOfExecutable;
+
     if (objc != 2) {
         Tcl_WrongNumArgs(interp, 2, objv, NULL);
         return TCL_ERROR;
     }
+
+    nameOfExecutable = Tcl_GetNameOfExecutable();
     
-    if (tclExecutableName != NULL) {
-	Tcl_SetStringObj(Tcl_GetObjResult(interp), tclExecutableName, -1);
+    if (nameOfExecutable != NULL) {
+	Tcl_SetStringObj(Tcl_GetObjResult(interp), (char *)nameOfExecutable, -1);
     }
     return TCL_OK;
 }
@@ -1633,13 +1659,13 @@ InfoVarsCmd(dummy, interp, objc, objv)
     char *varName, *pattern, *simplePattern;
     register Tcl_HashEntry *entryPtr;
     Tcl_HashSearch search;
-    Var *varPtr, *localVarPtr;
+    Var *varPtr;
     Namespace *nsPtr;
     Namespace *globalNsPtr = (Namespace *) Tcl_GetGlobalNamespace(interp);
     Namespace *currNsPtr   = (Namespace *) Tcl_GetCurrentNamespace(interp);
     Tcl_Obj *listPtr, *elemObjPtr;
     int specificNsInPattern = 0;  /* Init. to avoid compiler warning. */
-    int i, result;
+    int result;
 
     /*
      * Get the pattern and find the "effective namespace" in which to
@@ -1747,49 +1773,7 @@ InfoVarsCmd(dummy, interp, objc, objv)
 	    }
 	}
     } else {
-	/*
-	 * We're in a local call frame and no specific namespace was
-	 * specific. Create a list that starts with the compiled locals
-	 * (i.e. the ones stored in the call frame).
-	 */
-
-	CallFrame *varFramePtr = iPtr->varFramePtr;
-        int localVarCt = varFramePtr->numCompiledLocals;
-	Tcl_HashTable *varTablePtr = varFramePtr->varTablePtr;
-	
-        for (i = 0, localVarPtr = iPtr->varFramePtr->compiledLocals;
-                i < localVarCt;
-                i++, localVarPtr++) {
-            if (!TclIsVarUndefined(localVarPtr)) {
-                varName = localVarPtr->name;
-                if ((simplePattern == NULL)
-		        || Tcl_StringMatch(varName, simplePattern)) {
-                    Tcl_ListObjAppendElement(interp, listPtr,
-			    Tcl_NewStringObj(varName, -1));
-                }
-            }
-        }
-
-	/*
-	 * Now add in the variables in the call frame's variable hash
-	 * table (if one exists).
-	 */
-
-	if (varTablePtr != NULL) {
-	    for (entryPtr = Tcl_FirstHashEntry(varTablePtr, &search);
-		    entryPtr != NULL;
-		    entryPtr = Tcl_NextHashEntry(&search)) {
-		varPtr = (Var *) Tcl_GetHashValue(entryPtr);
-		if (!TclIsVarUndefined(varPtr)) {
-		    varName = Tcl_GetHashKey(varTablePtr, entryPtr);
-		    if ((simplePattern == NULL)
-		            || Tcl_StringMatch(varName, simplePattern)) {
-			Tcl_ListObjAppendElement(interp, listPtr,
-				Tcl_NewStringObj(varName, -1));
-		    }
-		}
-	    }
-	}
+	AppendLocals(interp, listPtr, simplePattern, 1);
     }
     
     Tcl_SetObjResult(interp, listPtr);
@@ -2937,7 +2921,7 @@ DictionaryCompare(left, right)
 	    diff = 0;
 	    while (1) {
 		if (diff == 0) {
-		    diff = *left - *right;
+		    diff = UCHAR(*left) - UCHAR(*right);
 		}
 		right++;
 		left++;
@@ -2961,17 +2945,17 @@ DictionaryCompare(left, right)
 	    }
 	    continue;
 	}
-        diff = *left - *right;
+        diff = UCHAR(*left) - UCHAR(*right);
         if (diff) {
             if (isupper(UCHAR(*left)) && islower(UCHAR(*right))) {
-                diff = tolower(*left) - *right;
+                diff = UCHAR(tolower(*left)) - UCHAR(*right);
                 if (diff) {
 		    return diff;
                 } else if (secondaryDiff == 0) {
 		    secondaryDiff = -1;
                 }
             } else if (isupper(UCHAR(*right)) && islower(UCHAR(*left))) {
-                diff = *left - tolower(UCHAR(*right));
+                diff = UCHAR(*left) - UCHAR(tolower(UCHAR(*right)));
                 if (diff) {
 		    return diff;
                 } else if (secondaryDiff == 0) {
