@@ -874,11 +874,11 @@ SetListFromAny(interp, objPtr)
     Tcl_Obj *objPtr;		/* The object to convert. */
 {
     Tcl_ObjType *oldTypePtr = objPtr->typePtr;
-    CONST char *elemStart, *nextElem;
     char *string, *s;
+    const char *elemStart, *nextElem;
     int lenRemain, length, estCount, elemSize, hasBrace, i, j, result;
     char *limit;		/* Points just after string's last byte. */
-    register CONST char *p;
+    register const char *p;
     register Tcl_Obj **elemPtrs;
     register Tcl_Obj *elemPtr;
     List *listRepPtr;
