@@ -12,7 +12,7 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
-# SCCS: @(#) safe.tcl 1.26 97/08/21 11:57:20
+# SCCS: @(#) safe.tcl 1.2 98/06/27 18:11:25
 
 #
 # The implementation is based on namespaces. These naming conventions
@@ -417,7 +417,7 @@ proc ::safe::interpAddToAccessPath {slave path} {
 
 	Lappend [VirtualPathListName $slave] $token;
 	Lappend [PathListName $slave] $path;
-	Set $nname [expr $n+1];
+	Set $nname [expr {$n+1}];
 
 	SyncAccessPath $slave;
 
@@ -532,7 +532,7 @@ proc ::safe::interpDelete {slave} {
 		# remove the hook now, otherwise if the hook
 		# calls us somehow, we'll loop
 		Unset $hookname;
-		if {[catch {eval $hook $slave} err]} {
+		if {[catch {eval $hook [list $slave]} err]} {
 		    Log $slave "Delete hook error ($err)";
 		}
 	    }
@@ -685,7 +685,7 @@ proc ::safe::setLogCmd {args} {
 	if {[regexp {(::)|(\.\.)} $path]} {
 	    error "invalid characters in path $path";
 	}
-	set n [expr [Set [PathNumberName $slave]]-1];
+	set n [expr {[Set [PathNumberName $slave]]-1}];
 	for {} {$n>=0} {incr n -1} {
 	    # fill the token virtual names with their real value
 	    set [PathToken $n] [Set [PathToken $n $slave]];
